
#include <Astri_DAQ.h>
#include <Astri_Redis_DAQ_Observer.h>
#include <Astri_MySQL_DAQ_Observer.h>
#include <CL_Conf.h>
#include <Mysql_Conf.h>
#include <Yaml_Conf.h>

#include <csignal>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ctime>
#include <iomanip>

#ifndef ASTRIDAQ_CONFIG_PATH
#define ASTRIDAQ_CONFIG_PATH std::string(std::getenv("HOME")).append("/BIAS/config/AstriDAQ")
#endif

using namespace inaf::oasbo;
using namespace inaf::oasbo::DAQ;
using namespace inaf::oasbo::DAQ_observers;

AstriDAQ *daq;

void my_handler(int s) {
	time_t now = time(nullptr);
	std::cout << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
			<< "]\t[Astri DAQ]" << std::endl;
	daq->getMonitorPtr()->printStats();
	std::cout << "-------------------------------" << std::endl;

	now = time(nullptr);
	std::cout << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
			<< "]\t[Astri DAQ]\t" << "Termination..." << std::endl;
	daq->stop();
}

/**
 * @brief The main function of the program.
 *
 * This function is the entry point of the program. It initializes the necessary configurators,
 * creates an instance of AstriDAQ, registers observers, starts the data acquisition process,
 * and waits until the acquisition is stopped. Finally, it terminates the program and cleans up
 * the resources.
 *
 * @param argc The number of command-line arguments.
 * @param argv An array of command-line arguments.
 * @return The exit status of the program.
 */
int main(int argc, char **argv) {
	Configurators::YamlConfigurator daqyamlconf(
			std::string(ASTRIDAQ_CONFIG_PATH).append("/config.yaml"));
	Configurators::MysqlConfigurator mysqlConf("127.0.0.1", 3306, "bias",
			"BIAS1ast01$1", "BIAS");
	Configurators::CLConfigurator clConf(argc, argv);

	daq =
			AstriDAQBuilder().configFrom(daqyamlconf)->configFrom(mysqlConf)->configFrom(
					clConf)->getDAQ();

	std::map<std::string, std::string> actConf;
	for (auto [k, v] : daqyamlconf.getConfig())
		actConf[k] = v;
	for (auto [k, v] : mysqlConf.getConfig())
		actConf[k] = v;
	for (auto [k, v] : clConf.getConfig())
		actConf[k] = v;

	time_t now = time(nullptr);
	std::cout << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
			<< "]\t[Astri DAQ]\t" "Starting program with params: " << std::endl;
	for (auto [k, v] : actConf)
		std::cout << k << " : " << v << std::endl;
	std::cout << "-------------------------------" << std::endl;

	AstriRedisDAQObserver obs(*daq, "127.0.0.1", 6379);
	daq->registerObserver(&obs);
	AstriMySQLDAQObserver mysql_obs(*daq, "127.0.0.1", 3306, "bias",
			"BIAS1ast01$1", "BIAS");
	daq->registerObserver(&mysql_obs);

	std::signal(SIGINT, [](int signal) {
		my_handler(signal);
	});
	std::signal(SIGTERM, [](int signal) {
		my_handler(signal);
	});


	daq->start();

	while(daq->getCurrentState() != AstriDAQ::Status::STOP){
		std::this_thread::sleep_for(std::chrono::seconds(1));
	}

	now = time(nullptr);
	std::cout << "[" << std::put_time(localtime(&now), "%Y-%m-%d %H:%M:%S")
			<< "]\t[Astri DAQ]\t" "Terminated normally" << std::endl;
	delete daq;

	return EXIT_SUCCESS;
}
